use rusqlite::{params, Connection, OptionalExtension, Result};

pub fn init_db() -> Result<Connection> {
    let conn = Connection::open("form_data.db")?;

    conn.execute(
        "CREATE TABLE IF NOT EXISTS forms (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL,
            fields TEXT NOT NULL,
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP
        )",
        [],
    )?;

    conn.execute(
        "CREATE TABLE IF NOT EXISTS submissions (
            id TEXT PRIMARY KEY,
            form_id TEXT NOT NULL,
            data TEXT NOT NULL,
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (form_id) REFERENCES forms (id) ON DELETE CASCADE
        )",
        [],
    )?;

    // Add a table for users
    conn.execute(
        "CREATE TABLE IF NOT EXISTS users (
            id TEXT PRIMARY KEY,
            username TEXT NOT NULL UNIQUE,
            password TEXT NOT NULL, -- Store a hashed password
            token TEXT UNIQUE       -- Optional: For token-based auth
        )",
        [],
    )?;

    Ok(conn)
}

// Add a function to validate a token
pub fn validate_token(conn: &Connection, token: &str) -> Result<Option<String>> {
    let mut stmt = conn.prepare("SELECT id FROM users WHERE token = ?1")?;
    let user_id: Option<String> = stmt
        .query_row(params![token], |row| row.get(0))
        .optional()?;
    Ok(user_id)
}

// Add a function to authenticate users (for login)
pub fn authenticate_user(
    conn: &Connection,
    username: &str,
    password: &str,
) -> Result<Option<String>> {
    let mut stmt = conn.prepare("SELECT id, password FROM users WHERE username = ?1")?;
    let mut rows = stmt.query(params![username])?;

    if let Some(row) = rows.next()? {
        let user_id: String = row.get(0)?;
        let stored_password: String = row.get(1)?;

        // Replace this with a secure password hashing and verification mechanism
        if stored_password == password {
            return Ok(Some(user_id));
        }
    }
    Ok(None)
}

// Add a function to generate and save a token for a user
pub fn generate_token_for_user(conn: &Connection, user_id: &str, token: &str) -> Result<()> {
    conn.execute(
        "UPDATE users SET token = ?1 WHERE id = ?2",
        params![token, user_id],
    )?;
    Ok(())
}
